
/* clang-print.h */

#ifndef __CLANG_PRINT_H__
#define __CLANG_PRINT_H__

#include "clang/AST/AST.h"

#include "htmlio.h"

OPEN_NAMESPACE

/* ---------------------------------------------------------------------- */

class TScanner : public THtmlOutput
{
   private:
      clang::ASTContext* fCtx;

   private:
      // only for debugging
      clang::Decl * fLastDecl;

   private:
      string GetSrcLocation (clang::SourceLocation L);
      string GetLocation (clang::Decl* D);
      string GetName (clang::Decl * D);

      void ShowInfo (string msg, string location);
      void ShowWarning (string msg, string location);

      void DeclInfo (clang::Decl * D);

      void PutQualifier (const string txt);

      void UnknownDecl (clang::Decl * D, string txt = "");
      void UnknownType (clang::QualType qual_type);

      // void ScanType (clang::Type * T);

      void ScanPointerType (const clang::PointerType * T);
      void ScanLValueReferenceType (const clang::LValueReferenceType * T);
      void ScanRValueReferenceType (const clang::RValueReferenceType * T);
      void ScanMemberPointerType (const clang::MemberPointerType * T);
      void ScanArrayType (const clang::ArrayType * T);
      void ScanFunctionType (const clang::FunctionType * T);
      void ScanQualType (clang::QualType qual_type);

      void ScanExpr (clang::Expr * expr);

      void ScanStorageClass (clang::VarDecl * D);
      void ScanVariable (clang::VarDecl * D);

      void ScanFuncModifiers (clang::FunctionDecl * D);
      void ScanFuncParameters (clang::FunctionDecl * D);
      void ScanFuncExceptions (const clang::FunctionProtoType * D);
      void ScanConstructorInitializer (clang::FunctionDecl * D);
      void ScanFunction (clang::FunctionDecl * D);

      void ScanEnum (clang::EnumDecl * D);

      void ScanVisibilityModifiers (clang::AccessSpecifier access);
      void ScanAccessSpecDecl (clang::AccessSpecDecl * D);
      void ScanFriendDecl (clang::FriendDecl * D);
      void ScanClass (clang::RecordDecl * D);

      void ScanTemplateParameter (clang::NamedDecl * D);
      void ScanTemplateParameterList (clang::TemplateParameterList * list);
      void ScanTemplate (clang::TemplateDecl * D);

      void ScanUsing (clang::UsingDecl * D);
      void ScanUsingDirective (clang::UsingDirectiveDecl * D);
      void ScanUnresolvedUsingValueMember (clang::UnresolvedUsingValueDecl * D);

      void ScanNamespace (clang::NamespaceDecl * D);
      void ScanNamespaceAlias (clang::NamespaceAliasDecl * D);
      void ScanLinkageSpec (clang::LinkageSpecDecl * D);
      void ScanTypedef (clang::TypedefDecl * D);
      void ScanTranslationUnit (clang::TranslationUnitDecl * D);

      void ScanDecl (clang::Decl * D);

   public:
      TScanner ();
      virtual ~ TScanner ();

      void Scan (clang::ASTContext * C, clang::Decl * D);
};

/* ---------------------------------------------------------------------- */

CLOSE_NAMESPACE

#endif /* __CLANG_PRINT_H__ */
