
/* plugin.cc */

#include "clang/Frontend/FrontendPluginRegistry.h"
#include "clang/AST/ASTConsumer.h"
#include "clang/AST/AST.h"
#include "clang/Frontend/CompilerInstance.h"
#include "llvm/Support/raw_ostream.h"
using namespace clang;

#include "clang/AST/ASTConsumer.h"

#include "clang-print.h"
#include "inf.h"

/* ---------------------------------------------------------------------- */

std::string OutputFile;

/* ---------------------------------------------------------------------- */

class ClrViewConsumer :  public ASTConsumer
{
  public:
    explicit ClrViewConsumer () {}

    NAMESPACE::TScanner scanner;

    void Initialize(ASTContext &Context)
    {
       scanner.Open (OutputFile);
       scanner.Head ("");
       scanner.PutLn("<pre>");

       // NAMESPACE::set_error_handlers ();
       //: ClrInit (&Context, ClrPrintFile, ClrStyleFile, ClrDictFile);
    }

    /*
    virtual void HandleTopLevelDecl(DeclGroupRef D)
    {
       for (clang::DeclGroupRef::iterator I = D.begin(), E = D.end(); I != E; ++I)
          ClrSend (*I);
    }
    */

    virtual void HandleTranslationUnit(ASTContext & Ctx)
    {
      for (DeclContext::decl_iterator D = Ctx.getTranslationUnitDecl()->decls_begin(),
           DEnd = Ctx.getTranslationUnitDecl()->decls_end();
           D != DEnd;
           ++D )
      {
         scanner.Scan (&Ctx, *D);
      }

      scanner.PutLn("</pre>");
      scanner.Tail ();
      scanner.WriteText ();
    }
};

/* ---------------------------------------------------------------------- */

class ClrViewAction : public PluginASTAction
{
   protected:
     virtual  std::unique_ptr<ASTConsumer> CreateASTConsumer (CompilerInstance &CI, llvm::StringRef InFile)
     {
        llvm::errs() << "in CreateASTConsumer" << "\n";
        return std::unique_ptr<ASTConsumer> (new ClrViewConsumer ());
     }

     bool ParseArgs (const CompilerInstance &CI,
                     const std::vector<std::string>& args)
     {
        unsigned int count = args.size ();
        llvm::errs() << "ParseArgs " << count << "\n";
        for (unsigned int i = 0; i < count; i++)
        {
           std::string s = args [i];
           llvm::errs() << "ClrView arg = " << s << "\n";

           if (s.substr(0,12) == "output-file=")
           {
              OutputFile = s.substr (12);
           }
           else if (s == "help")
           {
              PrintHelp (llvm::errs());
              return false;
           }
           else
           {
              PrintHelp (llvm::errs());
              DiagnosticsEngine &D = CI.getDiagnostics();
              unsigned DiagID = D.getCustomDiagID (DiagnosticsEngine::Error, "invalid argument '%0'");
              D.Report(DiagID) << args [i];
              return false;
           }
         }

        llvm::errs() << "OutputFile=" << OutputFile << ".\n";
        return true;
    }


    void PrintHelp (llvm::raw_ostream & io)
    {
       const char eol = '\n';

       io << eol;
       io << "llvm-plugin - Build ASTs and print HTML " << eol;
       io << eol;

       io << "-plugin-arg-llvm-plugin output-file=file-name  Specify output file" << eol;

       io << eol;
    }
};

/* ---------------------------------------------------------------------- */

static FrontendPluginRegistry::Add<ClrViewAction>
Reg ("llvm-plugin", "CLang to Reflex");
